(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udynamic_array.mli,v 1.5 2014/01/16 13:35:38 fclement Exp $ *)

(** {3 Dynamic arrays} *)

(** Dynamic arrays are arrays that may grow.
 When growing, new elements are initialized with the default value.
 Indices are natural numbers. *)

type index = Unatural.t;;
type length = Unatural.t;;

type 'a t =
  {
   mutable array : 'a array;
   default : 'a;
  }
;;

val same : 'a t -> 'a t -> bool;;

val max_length : length;;
val length : 'a t -> length;;

val make : length -> 'a -> 'a t;;

val get : 'a t -> index -> 'a;;
val set : 'a t -> index -> 'a -> unit;;

(** {6 Converters} *)

val to_array : 'a t -> 'a array;;
val of_array : 'a -> 'a array -> 'a t;;

(** {6 Iterators} *)

val iter : (index -> 'a -> unit) -> 'a t -> unit;;
val map : ( index -> 'a -> 'b) -> 'a t -> 'b t;;
val fold_left : ('accu -> index -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu;;
