(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id : Exp $ *)

type 'a t = { mutable contents : 'a option array; };;

let max_length = Unatural.of_int (Sys.max_array_length / 2);;

let make n = { contents = Uarray.make n None; };;

let get { contents = a; } i =
  match a.(Unatural.to_int i) with
  | None -> raise Not_found
  | Some x -> x
;;

let set ({ contents = a; } as dyn_a) i x =
  let len = Uarray.length a in
  if i < len then a.(Unatural.to_int i) <- Some x else
  let new_len = max (Unatural.succ i) (Unatural.mul Unatural._2_ len) in
  let new_len = if new_len > max_length then max_length else new_len in
  if new_len < i then
    failwith
      "Udynamic_array.set: cannot create such a large dynamic array" else
  let new_a = Uarray.make new_len None in
  Array.blit a 0 new_a 0 (Unatural.to_int len);
  new_a.(Unatural.to_int i) <- Some x;
  dyn_a.contents <- new_a
;;
